/*
 * @(#)ClientAgent.java
 * FileSharing package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
 
package ksu.cis.example.FileSharing;

import java.io.*;
import java.util.*;
import ksu.cis.mom.*;
import java.net.*;

/**
 * Client agent class. This agent request a file from server agent.
 * @author Chairoj Mekprasertvit
 *
 */
public class ClientAgent extends Agent
{
  private int state = 0;
  private boolean alive = true;
  public static final int unicastPort = 3000;
  private ClientAgentGUI gui;

  public ClientAgent(ClientAgentGUI gui)
  {
    super("ClientAgent", unicastPort);
    this.gui = gui;
    this.port = unicastPort;
    MessageHandler h = new MessageHandler(unicastPort, this);
    h.start();
  }
  /**
   * The performative that this agent can participate is is "found file".
   * Other performatives will be ignore.
   * 
   */
  public void receiveMessage(
    Socket server,
    ObjectInputStream input,
    ObjectOutputStream output)
  {
    Thread t;
    Message m;
    try
    {
      m = (Message) input.readObject();
      if(m.performative.equals("found file"))
      {
        t = new Thread(new ClientReceiveFile(server, input, output, this, m, 
          gui));
        t.start();         
      }  
    }
    catch(ClassNotFoundException cnfex)
    {
      System.out.println("** Class not found exception in " + this.name);
    }
    catch(IOException ioex)
    {
      System.out.println("** IOException in " + this.name);
    }
  }
  
  public void sendRequest(String req)
  {
    StringTokenizer token = new StringTokenizer(req);
    Thread init;
    init = new Thread(
        new ClientRequestFile(this, token.nextToken(), token.nextToken(), gui));
    init.start();
  }

  /* (non-Javadoc)
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    while(alive)
    {
      try
      {
        Thread.sleep(1000);
      }
      catch(InterruptedException iex)
      {
        System.out.println("** Thread Interrupted in " + this.name);
      }
    }
  }
}
