/*
 * @(#)ClientRequestFile.java
 * FileSharing package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
package ksu.cis.example.FileSharing;

import ksu.cis.mom.*;
import java.net.*;
import java.io.*;

/**
 * Unicast conversation initiator. 
 * @author Chairoj Mekprasertvit
 *
 */
public class ClientRequestFile extends Conversation
{
  private ClientAgent parent;
  private ClientAgentGUI gui;
  private String fileName;
  private String serverName;
  public ClientRequestFile(
    ClientAgent agent, 
    String server, 
    String req, 
    ClientAgentGUI gui)
  {
    super(agent, server, agent.port);
    parent = agent;
    fileName = req;
    serverName = server;
    this.gui = gui;
  }

  /**
   * 
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    Message m = new Message();
    int state = 0;
    boolean notDone = true;
    try
    {
      connection = new Socket(connectionHost, connectionPort);
      output = new ObjectOutputStream(connection.getOutputStream());
      output.flush();
      input = new ObjectInputStream(connection.getInputStream());
      while (notDone)
      {
        switch (state)
        {
          case 0 :
            m.performative = "find file";
            m.content = fileName;
            this.sendMessage(m, output);
            state = 1;
            break;
          case 1 :
            m = this.readMessage(input);
            if(m.performative.equals("reply"))
            {
              gui.setDisplay("The request is " + m.content.toString());
            }
            else
              parent.write("** ERROR - did not get reply back **");
            notDone = false;
            break;
        }
      }
      input.close();
      output.close();
      connection.close();
    }
    catch (UnknownHostException e)
    {
      parent.write("** Unknown Host exception in unicast. " + e.toString());
    }
    catch (EOFException oef)
    {
      parent.write(
        "** Server terminated connection in unicast. " + oef.toString());
    }
    catch (IOException e)
    {
      parent.write("** IO Exception in unicast. " + e.toString());
    }
  }
}
