/*
 * @(#)ClientAgentGUI.java
 * KeyDistribution package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
 
package ksu.cis.example.KeyDistribution;
import java.awt.*;
import java.awt.event.*;

/**
 * <p>Class ClientAgentGUI is a user interface for ClientAgent class that 
 * displays information on TextArea when agent receives message from other 
 * agents. The interface also has a TextField for user to type messages to 
 * send to other agents in the multicast group using secured multicast 
 * conversation.
 * @author Chairoj Mekprasertvit
 * @version 1.0, 01/20/2004
 */

public class ClientAgentGUI extends Frame
{
  private TextField field;
  private TextArea display;
  private ClientAgent agent;
  
  /**
   * Constrcutor for agent_interface.
   * @param name
   * @param broadcastAddress
   */
  public ClientAgentGUI(String name, String broadcastAddress)
  {
    super("Please Wait...");
    setLayout (new BorderLayout());
    field = new TextField();
    display = new TextArea();
    add(display, "Center");
    display.setBackground(Color.white);
    display.setForeground(Color.black);
    add(field, "South");
   
    setSize(300, 200);
    setVisible(true);
    show();
    
    agent = new ClientAgent(name, broadcastAddress, this);
    Thread t = new Thread(agent);
    t.start();

    //create field for outgoing secured multicast message.
    field.setEnabled(false);
    field.addActionListener
    (
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          setDisplay(e.getActionCommand());
          agent.sendSecuredMulticast(e.getActionCommand());
        }
      }
    );
  }
  
  public void startUI()
  {
    field.setEnabled(true);
    setTitle("Demo III: " + agent.name);
    setDisplay("** Client started.");
  }
  
  /**
   * <p>Enable the TextField. Allows user to type secured multicast message.
   */
  public void setField()
  {
    field.setEnabled(true);
  }
  
  public void setDisplay(String m)
  {
    display.append(m + "\n");
  }

  /**
   * main method
   * @param args
   */
  public static void main(String[] args)
  {
    if(args.length != 2)
    {
      System.out.println("Usage: java ClientAgentGUI <name> <broadcast address>");
      System.exit(1);
    }
    String name = args[0];
    String broadcastAddress = args[1];
    ClientAgentGUI gui = new ClientAgentGUI(name, broadcastAddress);
    gui.addWindowListener
    (
      new WindowAdapter()
      {
        public void windowClosing(WindowEvent e)
        {
          System.exit(0);
        }
      }
    );
    gui.startUI();
  }// end of main method
}//end of class agent_interface