/*
 * @(#)DistributorAgent.java
 * KeyDistribution package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */

package ksu.cis.example.KeyDistribution;

import java.io.*;
import java.net.*;
import java.security.*;
import javax.net.ssl.*;
import javax.crypto.*;
import ksu.cis.mom.*;
import java.util.*;

/**
 * <p>DistributorAgent class. This agent uses two types of communications.
 * <br>1. Broadcast to request register to ClientAgent.
 * <br>2. SecureUnicastMessage to send secret key to ClientAgent.
 * @author Chairoj Mekprasertvit
 * @version 1.0, 01/21/2004
 */

public class DistributorAgent extends Agent
{
  public static final int unicastPort = 3000;
  public static final int secureUnicastPort = 3001;
  public static final int multicastPort[] = {3002, 3003};
  public static final int broadcastPort = 3004;
  public static final int secureMulticastPort[] = {3005, 3006};
  private Vector clientList;
  private DistributorAgentGUI gui;
  private Key key;
  private String algorithm;
  private boolean alive;
  private boolean finishRegister;

  public DistributorAgent(
    String name, 
    String broadcastAddress, 
    DistributorAgentGUI gui)
  {
    super(
    name, 
    unicastPort, 
    multicastPort, 
    broadcastPort, 
    secureUnicastPort, 
    secureMulticastPort);
    this.gui = gui;
    this.alive = true;
    this.clientList = new Vector(10);
    this.finishRegister = false;
    
    try
    {
      super.group = new InetAddress[5];
      super.group[0] = InetAddress.getByName("239.255.10.11");
      super.broadcast_address = InetAddress.getByName(broadcastAddress);
      algorithm = "DES";
      key = KeyGenerator.getInstance(algorithm).generateKey();
      
    }
    catch (Throwable e)
    {
      write("Invalid Internet Address");
    }
    
    MessageHandler h = new MessageHandler(unicastPort, this);
    h.start();
    
    MulticastHandler mh =
      new MulticastHandler(this, multicastPort[0], 1, group[0]);
    mh.start();
    
    BroadcastHandler bh =
      new BroadcastHandler(this, broadcastPort, broadcast_address);
    bh.start();
    /*
    SecureUnicastHandler suh =
      new SecureUnicastHandler(secureUnicastPort, this);
    suh.start();*/
  }

  /**
   * 
   */
  public void receiveMessage(
    Socket server,
    ObjectInputStream input,
    ObjectOutputStream output)
  {

  }

  /**
   * <p>Receive message method for the broadcast conversation. Agent must 
   * override this method and defines all possible of secured multicast 
   * conversations here.<br>
   * This is the method that will be called by the <code>BroadcastHandler
   * </code> when the agent has received connection starting a new broadcast
   * conversation.</p>
   * @param bSocket - datagram socket for handling broadcast conversation
   * @param m - message from other agents
   * @param broadcast_queue - message queue for broadcast conversation
   */
  public void receiveBroadcastConversation(
    DatagramSocket bSocket,
    Message m,
    Vector broadcast_queue)
    {
      
    }

  /**
   * <p>Receive message method for the multicast conversation.
   * Agent must override this method and defines all possible of multicast 
   * conversations here.<br>
   * This is the method that will be called by the <code>MulticastHandler
   * </code> when the agent has received connection starting a new 
   * multicast conversation.</p>
   * @param mSocket multicast socket for handling multicast conversation
   * @param m Message from other agents
   * @param multicast_queue message queue for multicast conversation
   */
  public void receiveMulticastConversation(
    MulticastSocket mSocket,
    Message m,
    Vector multicast_queue)
    {
      
    }
    
  /**
   * <p>Receive multicast join message method for the multicast conversation.
   * Agent can override this method and performs some tasks. For example,
   * agent can keep track of other agents who join the group after itself.<br>
   * This is the method that will be called by the <code>MulticastHandler
   * </code> when the agent has received join message from other agents.</p>
   */
  public void receiveMulticastJoin(Message m)
  {
      
  }  
  
  /**
   * <p>Receive multicast leave message method for the multicast conversation.
   * Agent can override this method and performs some tasks. For example,
   * agent can keep track of other agents who leave.<br>
   * This is the method that will be called by the <code>MulticastHandler
   * </code> when the agent has received leave message from other agents.</p>
   */
  public void receiveMulticastLeave(Message m)
  {
      
  }
  
  /**
   * <p>Receive message method for the secured multicast conversation.
   * Agent must override this method and defines all possible of secured 
   * multicast conversations here.<br>
   * This is the method that will be called by the <code>SecureMulticastHandler
   * </code> when the agent has received connection starting a new secured
   * multicast conversation.</p>
   * @param mSocket - secured multicast socket for handling secured 
   * multicast conversation
   * @param m - message from other agents
   * @param multicast_queue - message queue for secured multicast conversation
   * @param k - symmetric private key.
   * @param algorithm - symmetric key algorithm name
   */
  public void receiveSecureMulticastConversation(
    MulticastSocket mSocket,
    Message m,
    Vector multicast_queue,
    Key k,
    String algorithm)
    {
      
    }

  /**
   * <p>Receive message method for the secured unicast conversation.
   * Agent must override this method and defines all possible of secured unicast 
   * conversations here.<br>
   * This is the method that will be called by the <code>SecureUnicastHandler
   * </code> when the agent has received connection starting a new 
   * secured unicast conversation.</p>
   * @param server - Secured socket for the secured unicast conversation
   * @param input - ObjectInputStream for the secured unicast conversation
   * @param output - ObjectOutputStream for the secured unicast conversation
   */
  public void receiveSecureUnicastConversation(
    SSLSocket server,
    ObjectInputStream input,
    ObjectOutputStream output)
    {
      
    }

  public void addToList(InetAddress ip)
  {
    this.clientList.addElement(ip);
  }
  
  public void setFinishRegister(boolean bool)
  {
    this.finishRegister = bool;
  }
  /**
   * 
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    int state = 0;
    Thread init = new Thread();
    while(alive)
    {
      switch(state)
      {
        case 0:
          gui.setDisplay("** Distributor sends Broadcast");
          init = new Thread(new DistributorRequestRegister(this, broadcastPort, broadcast_address, gui));
          init.start();
          state = 1;
          break;
        case 1:
          try
          {
            Thread.sleep(10000);
            if(finishRegister == true)
              state = 2;
            else
              state = 1;
          }
          catch(InterruptedException e)
          {
            System.out.println(e.toString());
          }
          break;
        case 2:
          for(int i=0; i<clientList.size(); i++)
          {
            gui.setDisplay("** Distributor sends secured unicast to all agents in the list");
            init = new Thread(new DistributorSendKey(
            this, (InetAddress)clientList.elementAt(i), secure_unicast_port, key, gui));
            init.start();
            gui.setDisplay("** Sending key to " + clientList.elementAt(i).toString());            
          }
          state = 3;
          break;
        case 3:
          try
          {
            gui.setDisplay("** Agent go to sleep before exit.");
            Thread.sleep(10000);
            if(finishRegister == true)
              System.exit(0);
            else
              state = 3;
            //System.exit(0);
          }
          catch(InterruptedException e)
          {
            System.out.println(e.toString());
          }
          break;
      }
    }
  }

}
