/*
 * Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 *
 */
package ksu.cis.example.KeyDistribution;

import ksu.cis.mom.*;
import java.net.*;
import java.io.*;

/**
 * @author Chairoj Mekprasertvit
 *
 */
public class DistributorRequestRegister extends BroadcastConversation
{
  private DistributorAgent parent;
  private String conversation_name;
  private DistributorAgentGUI gui;
  
  public DistributorRequestRegister(
    DistributorAgent agent,
    int b_port,
    InetAddress broadcast_address,
    DistributorAgentGUI gui)
    {
      super(agent, broadcast_address, b_port, agent.broadcast_queue);
      this.parent = agent;
      this.conversation_name = "DistributorRequestRegister";
      this.gui = gui;
    }

  /**
   * 
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    Message m = new Message();
    int state = 0;
    boolean notDone = true;
    int count = 0;
    try
    {
      while(notDone)
      {
        switch(state)
        {
          case 0:
            m.performative = "register";
            startConversation(m, conversation_name, "ClientReturnRegister");
            state = 1;
            break;
          case 1:
            try
            {
              gui.setDisplay("** Agent will wait for reply for 10000 millisec.");
              Thread.sleep(5000);
              gui.setDisplay("** 10000 milisec is over.");
            }
            catch(InterruptedException e)
            {
              System.out.println(e.toString());
            }
            state = 3;
            break;
          case 3:
          
            do
            {
              m = nonblockedReadMessage("DistributorRequestRegister", 1000);
              if(m.content.equals("timeout"))
              {
                parent.write("TIMEOUT BROADCAST");
                break;
              }

              if(m.performative.equals("register"))
              {
                parent.addToList((InetAddress) m.content);
                this.gui.setDisplay("IP: " + m.content.toString() + " is being registered.");
              }
            }
            while(!m.content.equals("timeout"));
            
          //m = readMessage("DistributorRequestRegister");
          //parent.addToList((InetAddress) m.content);
          //this.gui.setDisplay("IP: " + m.content.toString() + " is being registered.");

            notDone = false;
            parent.setFinishRegister(true);
            break;
        }
      }
    }
    catch(IOException e)
    {
      parent.write("** IO Exception in DistributorRegisterRequest conversation.");
    }
  }

}
