/*
 * @(#)AgentConversation.java
 * agentMom package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
package ksu.cis.mom;

/**
 * <p>This class is an abstract class that all types of conversation inherit 
 * from. It defines the minimum requirements for a conversation to be in 
 * agentMom package. It also allows user to easily implement a new type of
 * conversation for agentMom package.
 * Basically, this class is responsible for keeping track of all conversations
 * of the agent at a particular moment.
 * <p>History:<br>
 * 12/09/2003: creation date <br>
 * 12/20/2003: Add more comments suitable for JavaDoc.
 * @author Scott DeLoach and Chairoj Mekprasertvit
 * @version 1.0
 * @since agentMom 2.0
 *
 */

public abstract class AgentConversation extends Thread
{
  /**
   * Reference to parent agent class.
   */
  protected MomObject parent;
  
  /**
   * Port number of the host that this conversation is connected to.
   */
  protected int connectionPort;
  
  /**
   * Specify the host address
   */
  protected String connectionHost;
  
  /**
   * Message class is used for storing message sent from other agents.
   */
  protected Message m;
  
  /**
   * Default AgentConversation constructor.
   * @param c - Reference to parent MomObject class.
   */
  public AgentConversation(MomObject c)
  {
    super(c);
    parent = c;
  }
  
  /**
   * Constructor for conversation initiator.
   * @param c - Reference to parent MomObject class.
   * @param hostName - host name that this conversation is connected to.
   * @param portNum - port number of the host that this conversation is 
   * connected to.
   */
  public AgentConversation(MomObject c, String hostName, int portNum)
  {
    super(c);
    this.parent = c;
    this.connectionHost = hostName;
    this.connectionPort = portNum; 
  }

  /**
   * Constructor for conversation respondent.
   * @param c - Reference to parent MomObject class.
   * @param m ksu.cis.mom.Message
   */
  public AgentConversation(MomObject c, Message m)
  {
    super(c);
    this.parent = c;
    this.m = m;
  }

  /**
   * Method for easily printout information on screen.
   * @param mesg java.lang.String
   */
  public void write(String mesg)
  {
    System.out.println(mesg);
  }

}
