/*
 * @(#)MomObject.java
 * agentMom package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */

package ksu.cis.mom;

import java.io.*;
import java.net.*;

/**
 * <p>Abstract class that both Agents and Components inherit from. Allows 
 * conversations to work with either agents or conversations as their parents. 
 * Class that inherits from this class must implement the 
 * </code>sendInternal</code> method.
 * 
 * <p>History<br>
 * 05/3/2001: Creation date<br>
 * 08/23/2003: Add support for multicast, broadcast and sucurity features.<br>
 * 09/02/2003: Add more comments suitable for JavaDoc.</p>
 * @author Scott DeLoach and Chairoj Mekprasertvit
 * @version 2.0, 08/23/2003
 * @see ksu.cis.mom.Agent
 * @since agentMom 1.2
 */

public abstract class MomObject
	extends java.lang.Object
	implements Runnable, Serializable
{
  /**
   * 
   */
	public MomObject parent;
  
	/**
   * Name of the agent.
	 */
  public String name;
  
	/**
   * Port used for unicast conversation.
	 */
  public int port;
  
  /**
   * Array of port number used for multicast conversation. Note that each port
   * is used for each subscribed multicast group.
   */
  public int multicast_port[];
  
  /**
   * Port used for broadcast conversation.
   */
  public int broadcast_port;
  
  /**
   * Port used for secured unicast conversation.
   */
  public int secure_unicast_port;
  
  /**
   * Array of port number used for secured multicast conversation. Note that 
   * each port is used for each subscribed secured multicast group.
   */
  public int secure_multicast_port[];
  
  /**
   * Array of InetAddress type used for storing the address of multicast group.
   * Note that array is used to allow subscribing to multiple group.
   */
  public InetAddress group[];
  
  /**
   * Internet address of the broadcast address. Normally, local broadcast 
   * address is end with 255 (xxx.xxx.xxx.255). However, broadcast address is
   * not available in all network. Many networks do not permit the broadcast.
   * Please consult the admin about the availability. 
   */
  public InetAddress broadcast_address;
	
  /**
	 * <p>MomObject constructor. This constructor does not require any argument.
	 */
	public MomObject()
	{
		super();
	}
  
	/**
   * <p>MomObject constructor.
	 * @param c ksu.cis.MomObject
	 */
	public MomObject(MomObject c)
	{
		super();
		parent = c;
	}
  
  /**
	 * <p>Class that inherits from the MomObject class must implement the 
   * </code>sendInternal</code> method.</p>
   * @param m - message for internal conversation.
	 */
	public abstract void sendInternal(Message m);
}
