/*
 * @(#)Sorry.java
 * agentMom package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
package ksu.cis.mom;

import java.net.*;
import java.io.*;

/**
 * <p>The Sorry class defines a general purpose conversation to reply "Sorry"
 * to any unknown/unexpected type of unicast conversation. It is simply a 
 * default concrete class of conversation class.
 * <p>History<br>
 * 10/02/2003: Add more comments suitable for JavaDoc.<br>
 * 
 * @author Scott DeLoach and Chairoj Mekprasertvit
 * @version 2.0, 10/02/2003
 * @see ksu.cis.mom.Conversation
 * @since agentMom 1.0
 */

public class Sorry extends Conversation
{
	/**
   * <p>Sorry class constructor.
	 * @param s socket 
	 * @param i ObjectInputStream
	 * @param o ObjectOutputStream
	 * @param c MomObject
	 * @param m1 Message 
	 */
	public Sorry(
		Socket s,
		ObjectInputStream i,
		ObjectOutputStream o,
		MomObject c,
		Message m1)
	{
		super(c);
		parent = c;
		connection = s;
		input = i;
		output = o;
		m = m1;
	}

  /**
	 * <p>Run method for the Sorry class. It simply reply message with 
   * performative "sorry" and content "unknown conversation request". 
   * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		int state = 0;
		boolean notDone = true;

		//set up conversation
		try
		{
			while (notDone)
			{
				switch (state)
				{
					case 0 :
						m.setPerformative("sorry");
						m.setContent("unknown conversation request");
						sendMessage(m, output);
						notDone = false;
						break;
				}
			}
			input.close();
			output.close();
			connection.close();
		}
		catch (UnknownHostException e)
		{
			System.out.println(" ** Unknown Host exception.");
		}
		catch (EOFException oef)
		{
			System.out.println(" ** Server terminated connection.");
		}
		catch (IOException e)
		{
			System.out.println(" ** IO Exception.");
			e.printStackTrace();
		}
	}
}
